@@ -1124,9 +1124,9 @@ redraw_handler(struct widget *widget, void *data)
 				cairo_stroke(cr);
 			}
 
-                        /* skip space glyph (RLE) we use as a placeholder of
-                           the right half of a double-width character,
-                           because RLE is not available in every font. */
+			/* skip space glyph (RLE) we use as a placeholder of
+			   the right half of a double-width character,
+			   because RLE is not available in every font. */
 			if (p_row[col].ch == 0x200B)
 				continue;
 
@@ -2949,6 +2949,8 @@ terminal_create(struct display *display)
 	terminal->widget = window_frame_create(terminal->window, terminal);
 	terminal->title = xstrdup("Wayland Terminal");
 	window_set_title(terminal->window, terminal->title);
+	window_set_appid(terminal->window,
+			 "org.freedesktop.weston.wayland-terminal");
 	widget_set_transparent(terminal->widget, 0);
 
 	init_state_machine(&terminal->state_machine);
@@ -3138,12 +3140,12 @@ int main(int argc, char *argv[])
 
 	option_shell = getenv("SHELL");
 	if (!option_shell)
-		option_shell = "/bin/bash";
+		option_shell = "/bin/sh";
 
 	config_file = weston_config_get_name_from_env();
 	config = weston_config_parse(config_file);
 	s = weston_config_get_section(config, "terminal", NULL, NULL);
-	weston_config_section_get_string(s, "font", &option_font, "mono");
+	weston_config_section_get_string(s, "font", &option_font, "monospace");
 	weston_config_section_get_int(s, "font-size", &option_font_size, 14);
 	weston_config_section_get_string(s, "term", &option_term, "xterm");
 	weston_config_destroy(config);
