@@ -24,6 +24,7 @@
 #include "config.h"
 
 #include <wayland-server.h>
+#include <assert.h>
 
 #include <libweston/libweston.h>
 #include <libweston/zalloc.h>
@@ -48,14 +49,14 @@ weston_desktop_client_add_destroy_listener(struct weston_desktop_client *client,
 	wl_signal_add(&client->destroy_signal, listener);
 }
 
-static void
-weston_desktop_client_destroy(struct wl_resource *resource)
+void
+weston_desktop_client_destroy(struct weston_desktop_client *client)
 {
-	struct weston_desktop_client *client =
-		wl_resource_get_user_data(resource);
 	struct wl_list *list = &client->surface_list;
 	struct wl_list *link, *tmp;
 
+	assert(client->resource == NULL);
+
 	wl_signal_emit(&client->destroy_signal, client);
 
 	for (link = list->next, tmp = link->next;
@@ -71,6 +72,18 @@ weston_desktop_client_destroy(struct wl_resource *resource)
 	free(client);
 }
 
+static void
+weston_desktop_client_handle_destroy(struct wl_resource *resource)
+{
+	struct weston_desktop_client *client =
+		wl_resource_get_user_data(resource);
+
+	assert(client->resource == resource);
+	client->resource = NULL;
+
+	weston_desktop_client_destroy(client);
+}
+
 static int
 weston_desktop_client_ping_timeout(void *user_data)
 {
@@ -117,13 +130,12 @@ weston_desktop_client_create(struct weston_desktop *desktop,
 
 	if (dispatcher != NULL)
 		wl_resource_set_dispatcher(client->resource, dispatcher,
-					   weston_desktop_client_destroy, client,
-					   weston_desktop_client_destroy);
+					   weston_desktop_client_handle_destroy, client,
+					   weston_desktop_client_handle_destroy);
 	else
 		wl_resource_set_implementation(client->resource, implementation,
 					       client,
-					       weston_desktop_client_destroy);
-
+					       weston_desktop_client_handle_destroy);
 
 	display = wl_client_get_display(client->client);
 	loop = wl_display_get_event_loop(display);
