@@ -28,10 +28,10 @@
 
 #include <wayland-server.h>
 
-#include "compositor.h"
-#include "zalloc.h"
+#include <libweston/libweston.h>
+#include <libweston/zalloc.h>
 
-#include "libweston-desktop.h"
+#include <libweston-desktop/libweston-desktop.h>
 #include "internal.h"
 
 struct weston_desktop_view {
@@ -64,6 +64,7 @@ struct weston_desktop_surface {
 		char *title;
 		char *app_id;
 		pid_t pid;
+		struct wl_signal metadata_signal;
 	};
 	struct {
 		struct weston_desktop_surface *parent;
@@ -269,6 +270,8 @@ weston_desktop_surface_create(struct weston_desktop *desktop,
 	wsurface->committed = weston_desktop_surface_committed;
 	wsurface->committed_private = surface;
 
+	surface->pid = -1;
+
 	surface->surface_commit_listener.notify =
 		weston_desktop_surface_surface_committed;
 	wl_signal_add(&surface->surface->commit_signal,
@@ -285,6 +288,8 @@ weston_desktop_surface_create(struct weston_desktop *desktop,
 	wl_list_init(&surface->view_list);
 	wl_list_init(&surface->grab_link);
 
+	wl_signal_init(&surface->metadata_signal);
+
 	return surface;
 }
 
@@ -509,6 +514,13 @@ weston_desktop_surface_close(struct weston_desktop_surface *surface)
 					       surface->implementation_data);
 }
 
+WL_EXPORT void
+weston_desktop_surface_add_metadata_listener(struct weston_desktop_surface *surface,
+					     struct wl_listener *listener)
+{
+	wl_signal_add(&surface->metadata_signal, listener);
+}
+
 struct weston_desktop_surface *
 weston_desktop_surface_from_client_link(struct wl_list *link)
 {
@@ -590,7 +602,7 @@ weston_desktop_surface_get_pid(struct weston_desktop_surface *surface)
 {
 	pid_t pid;
 
-	if (surface->pid != 0) {
+	if (surface->pid != -1) {
 		pid = surface->pid;
 	} else {
 		struct weston_desktop_client *client =
@@ -598,6 +610,10 @@ weston_desktop_surface_get_pid(struct weston_desktop_surface *surface)
 		struct wl_client *wl_client =
 			weston_desktop_client_get_client(client);
 
+		/* wl_client should always be valid, because only in the
+		 * xwayland case it wouldn't be, but in that case we won't
+		 * reach here, as the pid is initialized to 0. */
+		assert(wl_client);
 		wl_client_get_credentials(wl_client, &pid, NULL, NULL);
 	}
 	return pid;
@@ -673,28 +689,32 @@ void
 weston_desktop_surface_set_title(struct weston_desktop_surface *surface,
 				 const char *title)
 {
-	char *tmp;
+	char *tmp, *old;
 
 	tmp = strdup(title);
 	if (tmp == NULL)
 		return;
 
-	free(surface->title);
+	old = surface->title;
 	surface->title = tmp;
+	wl_signal_emit(&surface->metadata_signal, surface);
+	free(old);
 }
 
 void
 weston_desktop_surface_set_app_id(struct weston_desktop_surface *surface,
 				  const char *app_id)
 {
-	char *tmp;
+	char *tmp, *old;
 
 	tmp = strdup(app_id);
 	if (tmp == NULL)
 		return;
 
-	free(surface->app_id);
+	old = surface->app_id;
 	surface->app_id = tmp;
+	wl_signal_emit(&surface->metadata_signal, surface);
+	free(old);
 }
 
 void
