@@ -33,6 +33,8 @@
 #include <assert.h>
 
 #include "pixman-renderer.h"
+#include "color.h"
+#include "pixel-formats.h"
 #include "shared/helpers.h"
 
 #include <linux/input.h>
@@ -125,20 +127,6 @@ pixman_renderer_read_pixels(struct weston_output *output,
 	return 0;
 }
 
-static void
-region_global_to_output(struct weston_output *output, pixman_region32_t *region)
-{
-	if (output->zoom.active) {
-		weston_matrix_transform_region(region, &output->matrix, region);
-	} else {
-		pixman_region32_translate(region, -output->x, -output->y);
-		weston_transformed_region(output->width, output->height,
-					  output->transform,
-					  output->current_scale,
-					  region, region);
-	}
-}
-
 #define D2F(v) pixman_double_to_fixed((double)v)
 
 static void
@@ -426,7 +414,8 @@ draw_view_translated(struct weston_view *view, struct weston_output *output,
 							  repaint_global,
 							  &surface->opaque,
 							  view);
-			region_global_to_output(output, &repaint_output);
+			weston_output_region_from_global(output,
+							 &repaint_output);
 
 			repaint_region(view, output, &repaint_output, NULL,
 				       PIXMAN_OP_SRC);
@@ -437,7 +426,7 @@ draw_view_translated(struct weston_view *view, struct weston_output *output,
 		region_intersect_only_translation(&repaint_output,
 						  repaint_global,
 						  &surface_blend, view);
-		region_global_to_output(output, &repaint_output);
+		weston_output_region_from_global(output, &repaint_output);
 
 		repaint_region(view, output, &repaint_output, NULL,
 			       PIXMAN_OP_OVER);
@@ -473,7 +462,7 @@ draw_view_source_clipped(struct weston_view *view,
 
 	pixman_region32_init(&repaint_output);
 	pixman_region32_copy(&repaint_output, repaint_global);
-	region_global_to_output(output, &repaint_output);
+	weston_output_region_from_global(output, &repaint_output);
 
 	repaint_region(view, output, &repaint_output, &buffer_region,
 		       PIXMAN_OP_OVER);
@@ -484,26 +473,31 @@ draw_view_source_clipped(struct weston_view *view,
 }
 
 static void
-draw_view(struct weston_view *ev, struct weston_output *output,
-	  pixman_region32_t *damage) /* in global coordinates */
+draw_paint_node(struct weston_paint_node *pnode,
+		pixman_region32_t *damage /* in global coordinates */)
 {
-	struct pixman_surface_state *ps = get_surface_state(ev->surface);
+	struct pixman_surface_state *ps = get_surface_state(pnode->surface);
 	/* repaint bounding region in global coordinates: */
 	pixman_region32_t repaint;
 
+	if (!pnode->surf_xform_valid)
+		return;
+
+	assert(pnode->surf_xform.transform == NULL);
+
 	/* No buffer attached */
 	if (!ps->image)
 		return;
 
 	pixman_region32_init(&repaint);
 	pixman_region32_intersect(&repaint,
-				  &ev->transform.boundingbox, damage);
-	pixman_region32_subtract(&repaint, &repaint, &ev->clip);
+				  &pnode->view->transform.boundingbox, damage);
+	pixman_region32_subtract(&repaint, &repaint, &pnode->view->clip);
 
 	if (!pixman_region32_not_empty(&repaint))
 		goto out;
 
-	if (view_transformation_is_translation(ev)) {
+	if (view_transformation_is_translation(pnode->view)) {
 		/* The simple case: The surface regions opaque, non-opaque,
 		 * etc. are convertible to global coordinate space.
 		 * There is no need to use a source clip region.
@@ -511,7 +505,7 @@ draw_view(struct weston_view *ev, struct weston_output *output,
 		 * Also the boundingbox is accurate rather than an
 		 * approximation.
 		 */
-		draw_view_translated(ev, output, &repaint);
+		draw_view_translated(pnode->view, pnode->output, &repaint);
 	} else {
 		/* The complex case: the view transformation does not allow
 		 * converting opaque etc. regions into global coordinate space.
@@ -520,7 +514,7 @@ draw_view(struct weston_view *ev, struct weston_output *output,
 		 * to be used whole. Source clipping does not work with
 		 * PIXMAN_OP_SRC.
 		 */
-		draw_view_source_clipped(ev, output, &repaint);
+		draw_view_source_clipped(pnode->view, pnode->output, &repaint);
 	}
 
 out:
@@ -530,11 +524,13 @@ static void
 repaint_surfaces(struct weston_output *output, pixman_region32_t *damage)
 {
 	struct weston_compositor *compositor = output->compositor;
-	struct weston_view *view;
+	struct weston_paint_node *pnode;
 
-	wl_list_for_each_reverse(view, &compositor->view_list, link)
-		if (view->plane == &compositor->primary_plane)
-			draw_view(view, output, damage);
+	wl_list_for_each_reverse(pnode, &output->paint_node_z_order_list,
+				 z_order_link) {
+		if (pnode->view->plane == &compositor->primary_plane)
+			draw_paint_node(pnode, damage);
+	}
 }
 
 static void
@@ -546,7 +542,7 @@ copy_to_hw_buffer(struct weston_output *output, pixman_region32_t *region)
 	pixman_region32_init(&output_region);
 	pixman_region32_copy(&output_region, region);
 
-	region_global_to_output(output, &output_region);
+	weston_output_region_from_global(output, &output_region);
 
 	pixman_image_set_clip_region32 (po->hw_buffer, &output_region);
 	pixman_region32_fini(&output_region);
@@ -571,6 +567,9 @@ pixman_renderer_repaint_output(struct weston_output *output,
 	struct pixman_output_state *po = get_output_state(output);
 	pixman_region32_t hw_damage;
 
+	assert(output->from_blend_to_output_by_backend ||
+	       output->from_blend_to_output == NULL);
+
 	if (!po->hw_buffer) {
 		po->hw_extra_damage = NULL;
  		return;
@@ -625,7 +624,7 @@ pixman_renderer_attach(struct weston_surface *es, struct weston_buffer *buffer)
 {
 	struct pixman_surface_state *ps = get_surface_state(es);
 	struct wl_shm_buffer *shm_buffer;
-	pixman_format_code_t pixman_format;
+	const struct pixel_format_info *pixel_info;
 
 	weston_buffer_reference(&ps->buffer_ref, buffer);
 	weston_buffer_release_reference(&ps->buffer_release_ref,
@@ -653,35 +652,24 @@ pixman_renderer_attach(struct weston_surface *es, struct weston_buffer *buffer)
 		return;
 	}
 
-	switch (wl_shm_buffer_get_format(shm_buffer)) {
-	case WL_SHM_FORMAT_XRGB8888:
-		pixman_format = PIXMAN_x8r8g8b8;
-		es->is_opaque = true;
-		break;
-	case WL_SHM_FORMAT_ARGB8888:
-		pixman_format = PIXMAN_a8r8g8b8;
-		es->is_opaque = false;
-		break;
-	case WL_SHM_FORMAT_RGB565:
-		pixman_format = PIXMAN_r5g6b5;
-		es->is_opaque = true;
-		break;
-	default:
+	pixel_info = pixel_format_get_info_shm(wl_shm_buffer_get_format(shm_buffer));
+	if (!pixel_info || !pixman_format_supported_source(pixel_info->pixman_format)) {
 		weston_log("Unsupported SHM buffer format 0x%x\n",
 			wl_shm_buffer_get_format(shm_buffer));
 		weston_buffer_reference(&ps->buffer_ref, NULL);
 		weston_buffer_release_reference(&ps->buffer_release_ref, NULL);
-                weston_buffer_send_server_error(buffer,
+		weston_buffer_send_server_error(buffer,
 			"disconnecting due to unhandled buffer type");
 		return;
-	break;
 	}
 
+	es->is_opaque = pixel_format_is_opaque(pixel_info);
+
 	buffer->shm_buffer = shm_buffer;
 	buffer->width = wl_shm_buffer_get_width(shm_buffer);
 	buffer->height = wl_shm_buffer_get_height(shm_buffer);
 
-	ps->image = pixman_image_create_bits(pixman_format,
+	ps->image = pixman_image_create_bits(pixel_info->pixman_format,
 		buffer->width, buffer->height,
 		wl_shm_buffer_get_data(shm_buffer),
 		wl_shm_buffer_get_stride(shm_buffer));
@@ -866,6 +854,8 @@ WL_EXPORT int
 pixman_renderer_init(struct weston_compositor *ec)
 {
 	struct pixman_renderer *renderer;
+	const struct pixel_format_info *pixel_info, *info_argb8888, *info_xrgb8888;
+	unsigned int i, num_formats;
 
 	renderer = zalloc(sizeof *renderer);
 	if (renderer == NULL)
@@ -891,7 +881,21 @@ pixman_renderer_init(struct weston_compositor *ec)
 		weston_compositor_add_debug_binding(ec, KEY_R,
 						    debug_binding, ec);
 
-	wl_display_add_shm_format(ec->wl_display, WL_SHM_FORMAT_RGB565);
+	info_argb8888 = pixel_format_get_info_shm(WL_SHM_FORMAT_ARGB8888);
+	info_xrgb8888 = pixel_format_get_info_shm(WL_SHM_FORMAT_XRGB8888);
+
+	num_formats = pixel_format_get_info_count();
+	for (i = 0; i < num_formats; i++) {
+		pixel_info = pixel_format_get_info_by_index(i);
+		if (!pixman_format_supported_source(pixel_info->pixman_format))
+			continue;
+
+		/* skip formats which libwayland registers by default */
+		if (pixel_info == info_argb8888 || pixel_info == info_xrgb8888)
+			continue;
+
+		wl_display_add_shm_format(ec->wl_display, pixel_info->format);
+	}
 
 	wl_signal_init(&renderer->destroy_signal);
 
