@@ -32,7 +32,7 @@
 #include <signal.h>
 #include <time.h>
 
-#include "shared/matrix.h"
+#include <libweston/matrix.h>
 
 struct inverse_matrix {
 	double LU[16];		/* column-major */
