@@ -67,6 +67,7 @@ struct weston_test {
 
 struct weston_test_surface {
 	struct weston_surface *surface;
+	struct wl_listener surface_destroy_listener;
 	struct weston_view *view;
 	int32_t x, y;
 	struct weston_test *test;
@@ -167,6 +168,84 @@ test_surface_committed(struct weston_surface *surface, int32_t sx, int32_t sy)
 	test_surface->view->is_mapped = true;
 }
 
+static int
+test_surface_get_label(struct weston_surface *surface, char *buf, size_t len)
+{
+	return snprintf(buf, len, "test suite surface");
+}
+
+static void
+test_surface_destroy(struct weston_test_surface *test_surface)
+{
+	weston_view_destroy(test_surface->view);
+
+	test_surface->surface->committed = NULL;
+	test_surface->surface->committed_private = NULL;
+	weston_surface_set_label_func(test_surface->surface, NULL);
+
+	wl_list_remove(&test_surface->surface_destroy_listener.link);
+	free(test_surface);
+}
+
+static void
+test_surface_handle_surface_destroy(struct wl_listener *l, void *data)
+{
+	struct weston_test_surface *test_surface =
+		wl_container_of(l, test_surface, surface_destroy_listener);
+
+	assert(test_surface->surface == data);
+
+	test_surface_destroy(test_surface);
+}
+
+static struct weston_test_surface *
+weston_test_surface_create(struct wl_resource *test_resource,
+			   struct weston_surface *surface)
+{
+	struct wl_client *client = wl_resource_get_client(test_resource);
+	struct wl_resource *display_resource;
+	struct weston_test_surface *test_surface;
+
+	test_surface = zalloc(sizeof *test_surface);
+	if (!test_surface)
+		goto err_post_no_mem;
+
+	test_surface->surface = surface;
+	test_surface->test = wl_resource_get_user_data(test_resource);
+
+	test_surface->view = weston_view_create(surface);
+	if (!test_surface->view)
+		goto err_free_surface;
+
+	/* Protocol does not define this error so abuse wl_display */
+	display_resource = wl_client_get_object(client, 1);
+	if (weston_surface_set_role(surface, "weston_test_surface",
+				    display_resource,
+				    WL_DISPLAY_ERROR_INVALID_OBJECT) < 0)
+		goto err_free_view;
+
+	surface->committed_private = test_surface;
+	surface->committed = test_surface_committed;
+	weston_surface_set_label_func(surface, test_surface_get_label);
+
+	test_surface->surface_destroy_listener.notify =
+		test_surface_handle_surface_destroy;
+	wl_signal_add(&surface->destroy_signal,
+		      &test_surface->surface_destroy_listener);
+
+	return test_surface;
+
+err_free_view:
+	weston_view_destroy(test_surface->view);
+
+err_free_surface:
+	free(test_surface);
+
+err_post_no_mem:
+	wl_resource_post_no_memory(test_resource);
+	return NULL;
+}
+
 static void
 move_surface(struct wl_client *client, struct wl_resource *resource,
 	     struct wl_resource *surface_resource,
@@ -175,28 +254,24 @@ move_surface(struct wl_client *client, struct wl_resource *resource,
 	struct weston_surface *surface =
 		wl_resource_get_user_data(surface_resource);
 	struct weston_test_surface *test_surface;
+	struct wl_resource *display_resource;
 
-	test_surface = surface->committed_private;
-	if (!test_surface) {
-		test_surface = malloc(sizeof *test_surface);
-		if (!test_surface) {
-			wl_resource_post_no_memory(resource);
-			return;
-		}
-
-		test_surface->view = weston_view_create(surface);
-		if (!test_surface->view) {
-			wl_resource_post_no_memory(resource);
-			free(test_surface);
-			return;
-		}
-
-		surface->committed_private = test_surface;
-		surface->committed = test_surface_committed;
+	if (surface->committed &&
+	    surface->committed != test_surface_committed) {
+		display_resource = wl_client_get_object(client, 1);
+		wl_resource_post_error(display_resource,
+				       WL_DISPLAY_ERROR_INVALID_OBJECT,
+				       "weston_test.move_surface: wl_surface@%u has a role.",
+				       wl_resource_get_id(surface_resource));
+		return;
 	}
 
-	test_surface->surface = surface;
-	test_surface->test = wl_resource_get_user_data(resource);
+	test_surface = surface->committed_private;
+	if (!test_surface)
+		test_surface = weston_test_surface_create(resource, surface);
+	if (!test_surface)
+		return;
+
 	test_surface->x = x;
 	test_surface->y = y;
 }
@@ -338,250 +413,6 @@ device_add(struct wl_client *client,
 	}
 }
 
-enum weston_test_screenshot_outcome {
-	WESTON_TEST_SCREENSHOT_SUCCESS,
-	WESTON_TEST_SCREENSHOT_NO_MEMORY,
-	WESTON_TEST_SCREENSHOT_BAD_BUFFER
-	};
-
-typedef void (*weston_test_screenshot_done_func_t)(void *data,
-						   enum weston_test_screenshot_outcome outcome);
-
-struct test_screenshot {
-	struct weston_compositor *compositor;
-	struct wl_global *global;
-	struct wl_client *client;
-	struct weston_process process;
-	struct wl_listener destroy_listener;
-};
-
-struct test_screenshot_frame_listener {
-	struct wl_listener listener;
-	struct weston_buffer *buffer;
-	struct weston_output *output;
-	weston_test_screenshot_done_func_t done;
-	void *data;
-};
-
-static void
-copy_bgra_yflip(uint8_t *dst, uint8_t *src, int height, int stride)
-{
-	uint8_t *end;
-
-	end = dst + height * stride;
-	while (dst < end) {
-		memcpy(dst, src, stride);
-		dst += stride;
-		src -= stride;
-	}
-}
-
-
-static void
-copy_bgra(uint8_t *dst, uint8_t *src, int height, int stride)
-{
-	/* TODO: optimize this out */
-	memcpy(dst, src, height * stride);
-}
-
-static void
-copy_row_swap_RB(void *vdst, void *vsrc, int bytes)
-{
-	uint32_t *dst = vdst;
-	uint32_t *src = vsrc;
-	uint32_t *end = dst + bytes / 4;
-
-	while (dst < end) {
-		uint32_t v = *src++;
-		/*                    A R G B */
-		uint32_t tmp = v & 0xff00ff00;
-		tmp |= (v >> 16) & 0x000000ff;
-		tmp |= (v << 16) & 0x00ff0000;
-		*dst++ = tmp;
-	}
-}
-
-static void
-copy_rgba_yflip(uint8_t *dst, uint8_t *src, int height, int stride)
-{
-	uint8_t *end;
-
-	end = dst + height * stride;
-	while (dst < end) {
-		copy_row_swap_RB(dst, src, stride);
-		dst += stride;
-		src -= stride;
-	}
-}
-
-static void
-copy_rgba(uint8_t *dst, uint8_t *src, int height, int stride)
-{
-	uint8_t *end;
-
-	end = dst + height * stride;
-	while (dst < end) {
-		copy_row_swap_RB(dst, src, stride);
-		dst += stride;
-		src += stride;
-	}
-}
-
-static void
-test_screenshot_frame_notify(struct wl_listener *listener, void *data)
-{
-	struct test_screenshot_frame_listener *l =
-		container_of(listener,
-			     struct test_screenshot_frame_listener, listener);
-	struct weston_output *output = l->output;
-	struct weston_compositor *compositor = output->compositor;
-	int32_t stride;
-	uint8_t *pixels, *d, *s;
-
-	weston_output_disable_planes_decr(output);
-	wl_list_remove(&listener->link);
-	stride = l->buffer->width * (PIXMAN_FORMAT_BPP(compositor->read_format) / 8);
-	pixels = malloc(stride * l->buffer->height);
-
-	if (pixels == NULL) {
-		l->done(l->data, WESTON_TEST_SCREENSHOT_NO_MEMORY);
-		free(l);
-		return;
-	}
-
-	/* FIXME: Needs to handle output transformations */
-
-	compositor->renderer->read_pixels(output,
-					  compositor->read_format,
-					  pixels,
-					  0, 0,
-					  output->current_mode->width,
-					  output->current_mode->height);
-
-	stride = wl_shm_buffer_get_stride(l->buffer->shm_buffer);
-
-	d = wl_shm_buffer_get_data(l->buffer->shm_buffer);
-	s = pixels + stride * (l->buffer->height - 1);
-
-	wl_shm_buffer_begin_access(l->buffer->shm_buffer);
-
-	/* XXX: It would be nice if we used Pixman to do all this rather
-	 *  than our own implementation
-	 */
-	switch (compositor->read_format) {
-	case PIXMAN_a8r8g8b8:
-	case PIXMAN_x8r8g8b8:
-		if (compositor->capabilities & WESTON_CAP_CAPTURE_YFLIP)
-			copy_bgra_yflip(d, s, output->current_mode->height, stride);
-		else
-			copy_bgra(d, pixels, output->current_mode->height, stride);
-		break;
-	case PIXMAN_x8b8g8r8:
-	case PIXMAN_a8b8g8r8:
-		if (compositor->capabilities & WESTON_CAP_CAPTURE_YFLIP)
-			copy_rgba_yflip(d, s, output->current_mode->height, stride);
-		else
-			copy_rgba(d, pixels, output->current_mode->height, stride);
-		break;
-	default:
-		break;
-	}
-
-	wl_shm_buffer_end_access(l->buffer->shm_buffer);
-
-	l->done(l->data, WESTON_TEST_SCREENSHOT_SUCCESS);
-	free(pixels);
-	free(l);
-}
-
-static bool
-weston_test_screenshot_shoot(struct weston_output *output,
-			     struct weston_buffer *buffer,
-			     weston_test_screenshot_done_func_t done,
-			     void *data)
-{
-	struct test_screenshot_frame_listener *l;
-
-	/* Get the shm buffer resource the client created */
-	if (!wl_shm_buffer_get(buffer->resource)) {
-		done(data, WESTON_TEST_SCREENSHOT_BAD_BUFFER);
-		return false;
-	}
-
-	buffer->shm_buffer = wl_shm_buffer_get(buffer->resource);
-	buffer->width = wl_shm_buffer_get_width(buffer->shm_buffer);
-	buffer->height = wl_shm_buffer_get_height(buffer->shm_buffer);
-
-	/* Verify buffer is big enough */
-	if (buffer->width < output->current_mode->width ||
-		buffer->height < output->current_mode->height) {
-		done(data, WESTON_TEST_SCREENSHOT_BAD_BUFFER);
-		return false;
-	}
-
-	/* allocate the frame listener */
-	l = malloc(sizeof *l);
-	if (l == NULL) {
-		done(data, WESTON_TEST_SCREENSHOT_NO_MEMORY);
-		return false;
-	}
-
-	/* Set up the listener */
-	l->buffer = buffer;
-	l->output = output;
-	l->done = done;
-	l->data = data;
-	l->listener.notify = test_screenshot_frame_notify;
-	wl_signal_add(&output->frame_signal, &l->listener);
-
-	/* Fire off a repaint */
-	weston_output_disable_planes_incr(output);
-	weston_output_schedule_repaint(output);
-
-	return true;
-}
-
-static void
-capture_screenshot_done(void *data, enum weston_test_screenshot_outcome outcome)
-{
-	struct wl_resource *resource = data;
-
-	switch (outcome) {
-	case WESTON_TEST_SCREENSHOT_SUCCESS:
-		weston_test_send_capture_screenshot_done(resource);
-		break;
-	case WESTON_TEST_SCREENSHOT_NO_MEMORY:
-		wl_resource_post_no_memory(resource);
-		break;
-	default:
-		break;
-	}
-}
-
-
-/**
- * Grabs a snapshot of the screen.
- */
-static void
-capture_screenshot(struct wl_client *client,
-		   struct wl_resource *resource,
-		   struct wl_resource *output_resource,
-		   struct wl_resource *buffer_resource)
-{
-	struct weston_output *output =
-		weston_head_from_resource(output_resource)->output;
-	struct weston_buffer *buffer =
-		weston_buffer_from_resource(buffer_resource);
-
-	if (buffer == NULL) {
-		wl_resource_post_no_memory(resource);
-		return;
-	}
-
-	weston_test_screenshot_shoot(output, buffer,
-				     capture_screenshot_done, resource);
-}
-
 static void
 send_touch(struct wl_client *client, struct wl_resource *resource,
 	   uint32_t tv_sec_hi, uint32_t tv_sec_lo, uint32_t tv_nsec,
@@ -608,7 +439,6 @@ static const struct weston_test_interface test_implementation = {
 	send_key,
 	device_release,
 	device_add,
-	capture_screenshot,
 	send_touch,
 };
 
@@ -750,7 +580,7 @@ static void
 idle_launch_testsuite(void *test_)
 {
 	struct weston_test *test = test_;
-	struct wet_testsuite_data *data = wet_testsuite_data_get();
+	struct wet_testsuite_data *data = weston_compositor_get_test_data(test->compositor);
 
 	if (!data)
 		return;
@@ -788,6 +618,8 @@ handle_compositor_destroy(struct wl_listener *listener,
 
 	test = wl_container_of(listener, test, destroy_listener);
 
+	wl_list_remove(&test->destroy_listener.link);
+
 	if (test->client_source) {
 		weston_log_scope_printf(test->log, "Cancelling client thread...\n");
 		pthread_cancel(test->client_thread);
